/***************************************************************************
                          guiview.h  -  description
 ***************************************************************************/

#ifndef GUIVIEW_H
#define GUIVIEW_H

#include "components/component.h"
#include "guidoc.h"
#include "viewpainter.h"
#include "guimain.h"

#include <qscrollview.h>
#include <qptrlist.h>
#include <qstring.h>
#include <qevent.h>
#include <qregexp.h>
#include <qvalidator.h>
#include <qlineedit.h>


class Wire;
class QWidget;
class QPainter;
class QPopupMenu;


// **************************************************************************
// ******* This class provides an incomplete base for the application *******
// ******* view. (scematics, data displays etc.)                      *******
// **************************************************************************

class QucsView : public QScrollView
{
  Q_OBJECT

public:
  QucsView(QWidget *parent=0);
  ~QucsView();

  float Zoom(float);
  bool   pasteElements();
  void   enlargeView(int, int, int, int);
  void   setPainter(QPainter*, QucsDoc*);
  void   eraseCross();
  void   editLabel(WireLabel*);

  Component *selComp;   // component selected in IconView
  Diagram   *selDiag;   // diagram selected in IconView
  Painting  *selPaint;  // painting selected in IconView

  bool    drawn;  // indicates whether the scheme element was drawn last time
  QString ProjName;
  QLineEdit *editText;  // for edit component properties

  QPtrList<Element_Qucs> movingElements;

  // menu appearing by right mouse button click on component
  QPopupMenu *ComponentMenu;

  // -------------------------------------------------------------------
  QPtrList<QucsDoc>  Docs; // document instances (schematics, data displays)

protected:
  void drawContents(QPainter*, int, int, int, int);
  void contentsMouseMoveEvent(QMouseEvent*);
  void contentsMousePressEvent(QMouseEvent*);
  void contentsMouseDoubleClickEvent(QMouseEvent*);
  void contentsMouseReleaseEvent(QMouseEvent*);
  void contentsWheelEvent(QWheelEvent*);

  bool ScrollUp(int);
  bool ScrollDown(int);
  bool ScrollLeft(int);
  bool ScrollRight(int);

protected slots:
  void slotScrollUp();
  void slotScrollDown();
  void slotScrollLeft();
  void slotScrollRight();
  void slotApplyCompText();

public slots:
  void slotCursorLeft();
  void slotCursorRight();
  void slotCursorUp();
  void slotCursorDown();

  void slotHideEdit();
  void slotEditElement();

public:
  void MouseDoNothing(QMouseEvent*);
  void MMoveSelect(QMouseEvent*);
  void MMoveComponent(QMouseEvent*);
  void MMoveDiagram(QMouseEvent*);
  void MMoveWire1(QMouseEvent*);
  void MMoveWire2(QMouseEvent*);
  void MMoveMoving(QMouseEvent*);
  void MMoveMoving2(QMouseEvent*);
  void MMovePaste(QMouseEvent*);
  void MMovePainting(QMouseEvent*);
  void MMoveDelete(QMouseEvent*);
  void MMoveLabel(QMouseEvent*);
  void MMoveMarker(QMouseEvent*);
  void MMoveMirrorY(QMouseEvent*);
  void MMoveMirrorX(QMouseEvent*);
  void MMoveRotate(QMouseEvent*);
  void MMoveActivate(QMouseEvent*);
  void MMoveOnGrid(QMouseEvent*);
  void MMoveResizePainting(QMouseEvent*);
  void MMoveMoveText(QMouseEvent*);
  void MMoveMoveTextB(QMouseEvent*);
  void MMoveZoomIn(QMouseEvent*);
  void (QucsView::*MouseMoveAction) (QMouseEvent*);// current mouse move method

  void MPressSelect(QMouseEvent*);
  void MPressDelete(QMouseEvent*);
  void MPressActivate(QMouseEvent*);
  void MPressMirrorX(QMouseEvent*);
  void MPressMirrorY(QMouseEvent*);
  void MPressRotate(QMouseEvent*);
  void MPressComponent(QMouseEvent*);
  void MPressDiagram(QMouseEvent*);
  void MPressLabel(QMouseEvent*);
  void MPressWire1(QMouseEvent*);
  void MPressWire2(QMouseEvent*);
  void MPressPainting(QMouseEvent*);
  void MPressMarker(QMouseEvent*);
  void MPressOnGrid(QMouseEvent*);
  void MPressMoveText(QMouseEvent*);
  void MPressZoomIn(QMouseEvent*);
  void (QucsView::*MousePressAction) (QMouseEvent*); // mouse press method

  void MDoubleClickSelect(QMouseEvent*);
  void MDoubleClickWire2(QMouseEvent*);
  void (QucsView::*MouseDoubleClickAction) (QMouseEvent*);

  void MReleaseSelect(QMouseEvent*);
  void MReleaseSelect2(QMouseEvent*);
  void MReleaseActivate(QMouseEvent*);
  void MReleaseMoving(QMouseEvent*);
  void MReleaseResizeDiagram(QMouseEvent*);
  void MReleasePaste(QMouseEvent*);
  void MReleaseResizePainting(QMouseEvent*);
  void MReleaseMoveText(QMouseEvent*);
  void MReleaseZoomIn(QMouseEvent*);
  void (QucsView::*MouseReleaseAction) (QMouseEvent*);

  void MovingElements();
  void endElementMoving();

private:
  void editElement(QMouseEvent*);
  void rightPressMenu(QMouseEvent*);
  void PressLabel(QMouseEvent*);

  int MAx1, MAy1,MAx2, MAy2, MAx3, MAy3;  // cache for mouse movements
  bool isMoveEqual;
  Element_Qucs *focusElement;
  QMouseEvent *focusMEvent;
  Wire *labeledWire;     // remember the wire whose label is moving
  ViewPainter Painter;

  QRegExp Expression;
  QRegExpValidator *Validator;
};

#endif
