	SUBROUTINE JACFD(JACTYP,N     ,NUNIT ,OUTPUT,EPSMCH,
     $                   BOUNDL,BOUNDU,FVECC ,FVECJ1,JACFDM,
     $                   SCALEX,WV3   ,XC    ,FVECEV)
C
C       FEB. 15, 1991
C
C       THIS SUBROUTINE EVALUATES THE JACOBIAN USING
C       ONE-SIDED FINITE DIFFERENCES.
C
C       JACTYP "1" SIGNIFIES FORWARD DIFFERENCES
C       JACTYP "2" SIGNIFIES BACKWARD DIFFERENCES
C
C       FVECJ1 IS A TEMPORARY VECTOR WHICH STORES THE RESIDUAL
C       VECTOR FOR THE FINITE DIFFERENCE CALCULATION.
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION  JACFDM(N,N)
        INTEGER           OUTPUT
	DIMENSION         BOUNDL(N)  ,BOUNDU(N) ,FVECC(N)  ,FVECJ1(N),
     $                    SCALEX(N)  ,WV3(N)    ,XC(N)
        LOGICAL           OVERFL     ,WRNSUP
        COMMON/NNES_2/WRNSUP
        EXTERNAL FVECEV
        DATA ZERO,ONE /0.0D0,1.0D0/
C
        SQRTEP=SQRT(EPSMCH)
C
C       FINITE-DIFFERENCE CALCULATION BY COLUMNS.
C
        DO 100 J=1,N
C
C          DELTAJ IS THE STEP SIZE - IT IS ALWAYS POSITIVE.
C
           DELTAJ=SQRTEP*MAX(ABS(XC(J)),ONE/SCALEX(J))
           TEMPJ=XC(J)  ! TEMPORARY STORAGE OF XC(J)
           IF(JACTYP.EQ.1) THEN
              IF(XC(J)+DELTAJ.LE.BOUNDU(J)) THEN
C
C                STEP WITHIN BOUNDS - COMPLETE FORWARD DIFFERENCE.
C
                 XC(J)=XC(J)+DELTAJ
		 DELTAJ=XC(J)-TEMPJ
		 CALL FORDIF(OVERFL,J     ,N     ,DELTAJ,FVECC ,
     $                       FVECJ1,JACFDM,XC    ,FVECEV)
              ELSE
C
C                STEP WOULD VIOLATE BOUNDU - TRY BACKWARD DIFFERENCE.
C
                 IF(XC(J)-DELTAJ.GE.BOUNDL(J)) THEN
                    XC(J)=XC(J)-DELTAJ
                    CALL BAKDIF(OVERFL,J     ,N     ,DELTAJ,TEMPJ ,
     $                          FVECC ,FVECJ1,JACFDM,XC    ,FVECEV)
                 ELSE
C
C                   STEP WOULD ALSO VIOLATE BOUNDL - IF THE DIFFERENCE
C                   IN THE BOUNDS, (BOUNDU-BOUNDL), IS GREATER THAN
C                   DELTAJ CALCULATE THE FUNCTION VECTOR AT EACH BOUND
C                   AND USE THIS DIFFERENCE - THIS REQUIRES ONE EXTRA
C                   FUNCTION EVALUATION. THE CURRENT FVECC IS STORED
C                   IN WV3, THEN REPLACED.
C
                    IF(BOUNDU(J)-BOUNDL(J).GE.DELTAJ) THEN
		       CALL BNDDIF(OVERFL,J     ,N     ,EPSMCH,BOUNDL,
     $                             BOUNDU,FVECC ,FVECJ1,JACFDM,WV3   ,
     $                             XC    ,FVECEV)
                       IF(OUTPUT.GT.2.AND.(.NOT.WRNSUP).AND.
     $                    (.NOT.OVERFL)) THEN
                          WRITE(NUNIT,1)
1                         FORMAT(T3,'*',T74,'*')
                          WRITE(NUNIT,2)
2                         FORMAT(T3,'*',4X,'WARNING: BOUNDS TOO CLOSE',
     $                    ' FOR 1-SIDED FINITE-DIFFERENCES',T74,'*')
                          WRITE(NUNIT,3) J
3                         FORMAT(T3,'*',13X,'LOWER AND UPPER BOUNDS',
     $                    ' USED FOR JACOBIAN COLUMN: ',I3,T74,'*')
                          WRITE(NUNIT,4)
4                         FORMAT(T3,'*',13X,'THIS REQUIRED ONE EXTRA',
     $                    ' FUNCTION EVALUATION',T74,'*')
                       END IF
		    ELSE
C
C                      BOUNDS ARE EXTREMELY CLOSE (BUT NOT EQUAL OR
C                      THE PROGRAM WOULD HAVE STOPPED IN INITCH).
C
                       IF(OUTPUT.GT.2.AND.(.NOT.WRNSUP).AND.
     $                    (.NOT.OVERFL)) THEN
                          WRITE(NUNIT,1)
                          WRITE(NUNIT,2)
                          WRITE(NUNIT,5) J
5                         FORMAT(T3,'*',13X,'BOUNDS ARE EXTREMELY',
     $                    ' CLOSE FOR COMPONENT: ',I3,T74,'*')
			  WRITE(NUNIT,6)
6                         FORMAT(T3,'*',13X,'FINITE DIFFERENCE',
     $                    ' JACOBIAN IS UNRELIABLE',T74,'*')
                       END IF
		       CALL BNDDIF(OVERFL,J     ,N     ,EPSMCH,BOUNDL,
     $                             BOUNDU,FVECC ,FVECJ1,JACFDM,WV3   ,
     $                             XC    ,FVECEV)
		    END IF
                 END IF
              END IF
           ELSE
              IF(XC(J)-DELTAJ.GE.BOUNDL(J)) THEN
                 XC(J)=XC(J)-DELTAJ
		 CALL BAKDIF(OVERFL,J     ,N     ,DELTAJ,TEMPJ ,FVECC ,
     $                       FVECJ1,JACFDM,XC    ,FVECEV)
              ELSE
                 IF(XC(J)+DELTAJ.LE.BOUNDU(J)) THEN
                    XC(J)=XC(J)+DELTAJ
		    CALL FORDIF(OVERFL,J     ,N     ,DELTAJ,
     $                          FVECC ,FVECJ1,JACFDM,XC    ,FVECEV)
                 ELSE
                    IF(BOUNDU(J)-BOUNDL(J).GE.DELTAJ) THEN
		       CALL BNDDIF(OVERFL,J     ,N     ,EPSMCH,BOUNDL,
     $                             BOUNDU,FVECC ,FVECJ1,JACFDM,WV3   ,
     $                             XC    ,FVECEV)
                       IF(OUTPUT.GT.2.AND.(.NOT.WRNSUP).AND.
     $                    (.NOT.OVERFL)) THEN
                          WRITE(NUNIT,1)
                          WRITE(NUNIT,2)
                          WRITE(NUNIT,3) J
                          WRITE(NUNIT,4)
                       END IF
                    ELSE
		       CALL BNDDIF(OVERFL,J     ,N     ,EPSMCH,BOUNDL,
     $                             BOUNDU,FVECC ,FVECJ1,JACFDM,WV3   ,
     $                             XC    ,FVECEV)
		       DO 300 I=1,N
                          JACFDM(I,J)=ZERO
300                    CONTINUE
                       IF(OUTPUT.GT.2.AND.(.NOT.WRNSUP).AND.
     $                    (.NOT.OVERFL)) THEN
                          WRITE(NUNIT,1)
                          WRITE(NUNIT,2)
			  WRITE(NUNIT,5) J
			  WRITE(NUNIT,6)
                       END IF
                    END IF
                END IF
              END IF
           END IF
           IF(OVERFL.AND.OUTPUT.GT.2.AND.(.NOT.WRNSUP)) THEN
              WRITE(NUNIT,1)
	      WRITE(NUNIT,7)
7             FORMAT(T3,'*',4X,'WARNING: OVERFLOW IN FUNCTION',
     $        ' VECTOR IN SUBROUTINE JACFD',T74,'*')
              WRITE(NUNIT,5) J
              DO 400 I=1,N
                 JACFDM(I,J)=ZERO
400           CONTINUE
           END IF
           XC(J)=TEMPJ
100     CONTINUE
        RETURN
C
C       LAST CARD OF SUBROUTINE JACFD.
C
        END
