#include "NLSInterface.h"

extern "C"
{
  // include this to use the symbol table
#include <cstdio>
#include "../compat/stuff.h"
  // use the report() function for messages
#include "../inout/report.h"

  // Fortran nonlinear equation solver routines (NNES)
    void setup_(int * ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,double* ,double* ,double* ,double* ,
	 double* ,double* ,double* ,double* ,double* ,
	 double* ,double* ,double* ,double* ,double* ,
	 double* ,double *,double *,double *,double *,
	 char *);

	     void nnesdr_(int * ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 int* ,int* ,double* ,int* ,int* ,
	 int* ,int* ,int* ,int* ,int* ,
	 double* ,double* ,double* ,double* ,double* ,
	 double* ,double* ,double* ,double* ,double* ,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,double *,
	 double* ,double *,double *,double *,char *,
	 void (*)(int*, int*, double*, double*) ,
	 void (*)(int*, int*, double*, double*),
	 void (*) ,void (*)
	 );
    double twonrm_(int *,int *,int *,double *,double *,double *);
    void incrnfetot_();
}

NLSInterface* NLSInterface::nlsi = NULL;

NLSInterface::NLSInterface(OFunction* of, int& sysdim, int diff)
{
  char msg[132];
  if (nlsi)
    report(FATAL, "Attempt to create 2 nonlinear solver interface intances.");

  nlsi = this;
  this->of = of;

  mgll = 10;
  n = sysdim;
  nunit = 6;
  matsup = 0;

  // Allocate space for vectors and arrays
  jac = new double[n * n];
  a = new double[n * n];
  boundl = new double[n];
  boundu = new double[n];
  delf = new double[n];
  fsave = new double[n];
  ftrack = new double[mgll];
  fvec = new double[n];
  fvecc = new double[n];
  h = new double[n * n];
  hhpi = new double[n];
  plee = new double[n * n];
  rdiag = new double[n];
  s = new double[n];
  sbar = new double[n];
  scalef = new double[n];
  scalex = new double[n];
  sn = new double[n];
  ssdhat = new double[n];
  strack = new double[mgll];
  vhat = new double[n];
  wv1 = new double[n];
  wv2 = new double[n];
  wv3 = new double[n];
  wv4 = new double[n];
  xplus = new double[n];
  xsave = new double[n];

  // Call the fortran routine to set default values.
  setup_(&absnew ,&cauchy ,&deuflh ,&geoms  ,&linesr ,
	 &newton ,&overch ,&acptcr ,&itsclf ,&itsclx ,
	 &jactyp ,&jupdm  ,&maxexp ,&maxit  ,&maxns  ,
	 &maxqns ,&minqns ,&n      ,&narmij ,&niejev ,
	 &njacch ,&output ,&qnupdm ,&stopcr ,&supprs ,
	 &trupdm ,&alpha  ,&confac ,&delta  ,&delfac ,
	 &epsmch ,&etafac ,&fdtolj ,&ftol   ,&lam0   ,
	 &mstpf  ,&nsttol ,&omega  ,&ratiof ,&sigma  ,
	 &stptol , boundl , boundu , scalef , scalex ,
	  help);

  // Variables to read symbol table
  generic_t gval;
  int type;
  // Set method
  if (St_GetSym(capOptionsT_P, "method", &type, &gval) == ST_SYM_FOUND)
  {
    if (type != GEN_TYPE_INT)
    {
      gval.i = 3;
      report(WARNING, "Option \"method\" should be an integer.");
    }
  }
  else
    gval.i = 3;

  switch (gval.i) {
  case 1:
    // set Newton
    report(MESSAGE, "Using plain Newton method.");
    newton = linesr = TRUE;
    absnew = FALSE;
    break;
  case 2:
    // Absolute Newton
    report(MESSAGE, "Using absolute Newton method.");
    newton = FALSE;
    absnew = linesr = TRUE;
    break;
  case 4:
    // Set trust region method
    report(MESSAGE, "Using trust region method.\n");
    newton = absnew = linesr = FALSE;
    break;
  default:
    // Set line search (default)
    report(MESSAGE, "Using line search method.");
  }

  // set bypass to false
  bypass = FALSE;

  // set the stop tolerances
  if (St_GetSym(capOptionsT_P, "ftol", &type, &gval) == ST_SYM_FOUND)
    if (type == GEN_TYPE_DOUBLE)
      ftol = gval.d;
    else if (type == GEN_TYPE_FLOAT)
      ftol = gval.f;
    else
      report(WARNING, "Option \"ftol\" should be float.");
  sprintf(msg,"Nonlinear analysis tolerance (ftol) = %g",ftol);
  report(MESSAGE,msg);

  // set the maximun number of iterations
  if (St_GetSym(capOptionsT_P, "maxit", &type, &gval) == ST_SYM_FOUND)
    if (type == GEN_TYPE_INT)
      maxit = gval.i;
    else
      report(WARNING, "Option \"maxit\" should be an integer.");
  sprintf(msg,
    "Maximum number of nonlinear iterations per time-point (maxit) = %d",maxit);
  report(MESSAGE,msg);

  // Check that jactyp is valid
  jactyp = diff;
  assert(jactyp > -1 || jactyp < 4);

  // Set Jacobian update type.
  jupdm = 2;
  if (St_GetSym(capOptionsT_P, "jupdm", &type, &gval) == ST_SYM_FOUND)
    if (type != GEN_TYPE_INT)
      report(WARNING, "Option \"jupdm\" should be an integer.");
    else
      jupdm = gval.i;

  switch (jupdm)
	{
  case 0:
    report(MESSAGE, "Jacobian evaluated at every iteration.");
    break;
  case 1:
    report(MESSAGE, "Using Broyden's quasi-Newton updates.");
    break;
  default:
    jupdm = 2;
    report(MESSAGE, "Using Lee and Lee's quasi-Newton updates.");
  }

  //  njacch = 1;
  // Do not check Jacobian
  njacch = 0;

  // set verbose level
  output = 0;
  if (St_GetSym(capOptionsT_P, "output", &type, &gval) == ST_SYM_FOUND)
    if (type == GEN_TYPE_INT)
      output = (gval.i > -1) ? gval.i : 0;
    else
      report(WARNING, "Option \"output\" should be an integer.");
}

NLSInterface::~NLSInterface()
{
  // free memory.
  delete [] xsave;
  delete [] xplus;
  delete [] wv4;
  delete [] wv3;
  delete [] wv2;
  delete [] wv1;
  delete [] vhat;
  delete [] strack;
  delete [] ssdhat;
  delete [] sn;
  delete [] scalex;
  delete [] scalef;
  delete [] sbar;
  delete [] s;
  delete [] rdiag;
  delete [] plee;
  delete [] hhpi;
  delete [] h;
  delete [] fvecc;
  delete [] fvec;
  delete [] ftrack;
  delete [] fsave;
  delete [] delf;
  delete [] boundu;
  delete [] boundl;
  delete [] a;
  delete [] jac;

  nlsi = NULL;
}

void nlfcn(int *overfl, int *n1, double* F, double* X)
{
  *overfl = FALSE;
  // Call the objective function routine
  NLSInterface::nlsi->of->func_ev(X, F);
  // Increment counter of evaluations
  incrnfetot_();

  return;
}

void jacFunc(int *overfl, int *n1, double* J, double* X)
{
  *overfl = FALSE;
  // Build a view of the Fortran matrix.
  DoubleDenseExtColMatrix myJ(&J[0], *n1, *n1);
  // Call the objective function jacobian routine
  NLSInterface::nlsi->of->jacobian(X, myJ);

  return;
}

int NLSInterface::solve(double* X, double& res)
{
  // Call the nonlinear equation solver.
  nnesdr_(&absnew ,&cauchy ,&deuflh ,&geoms  ,&linesr ,
	&newton ,&overch ,&acptcr ,&itsclf ,&itsclx ,
	&jactyp ,&jupdm  ,&maxexp ,&maxit  ,&maxns  ,
	&maxqns ,&mgll   ,&minqns ,&n      ,&narmij ,
	&niejev ,&njacch ,&njetot ,&nunit  ,&output ,
	&qnupdm ,&stopcr ,&supprs ,&trmcod ,&trupdm ,
	&alpha  ,&confac ,&delta  ,&delfac ,&epsmch ,
	&etafac ,&fcnnew ,&fdtolj ,&ftol   ,&lam0   ,
	&mstpf  ,&nsttol ,&omega  ,&ratiof ,&sigma  ,
	&stptol , a      , boundl , boundu , delf   ,
	fsave  , ftrack , fvec   , fvecc  , h      ,
	hhpi   , jac    , plee   , rdiag  , s      ,
	sbar   , scalef , scalex , sn     , ssdhat ,
	strack , vhat   , wv1    , wv2    , wv3    ,
	wv4    , X      , xplus  , xsave  , help   ,
	nlfcn  , jacFunc,
	&bypass, &matsup);

  // Calculate the final norm of the residuals and copy the result to X
  res = zero;
  if (trmcod)
	{
    for (int i=0; i < n; i++)
		{
			X[i] = xplus[i];
		}
	}

  nlfcn(&overfl, &n, fvec, X);
  twonrm_(&overfl,&maxexp,&n,&epsmch,&res,fvec);

  int retcod = (trmcod == 2) ? 0 : -1;
  // If the solution is basically correct, just let it go...
  if (retcod && res < 10.*ftol)
    retcod = 1;

  return(retcod);
}

