// Code with optimization with optional dc solution and
// efficient T'Ji multiplication and gcomp calculation
// Efficient SuperLUFactor no densevector's
#include "SVTran.h"
#include <iostream>
using namespace std;
//#include  "mvblasd.h"

extern int superLU_print_enable ;
extern "C"
{
	#include "../inout/ftvec.h"
	#include "../inout/report.h"
}

void buildTIncidence(ElemFlag mask, Circuit*& my_circuit,
IntDenseColMatrix& T, ElementVector& elem_vec,
int& n_states, int& max_n_states);

// Static members
const unsigned SVTran::n_par = 14;

// Element information
ItemInfo SVTran::ainfo =
{
  "SVTran",
  "State-Variable-Based Time-Marching Transient Analysis with Sparse matrices and Newton Iterations",
  "Sonali Luniya",
  DEFAULT_ADDRESS
};

// Parameter information
ParmInfo SVTran::pinfo[] =
{
  {"tstop", "Stop time (s)", TR_DOUBLE, true},
  {"tstep", "Time step (s)", TR_DOUBLE, true},
  {"nst", "No save time (s)", TR_DOUBLE, false},
  {"deriv", "Approximate derivatives or use automatic diff.", TR_INT, false},
  {"msv", "Use Msv flag", TR_BOOLEAN, false},
  {"im", "Integration method", TR_INT, false},
  {"savenode", "Save node voltages", TR_BOOLEAN, false},
  {"permc_spec", "Permutation ordering to factor Msv (0, 1 or 2)", TR_INT, false},
  {"out_steps", "Number of steps skipped for output simulation progress", TR_INT, false},
  {"gcomp", "Compensation network conductance (S)", TR_DOUBLE, false},
  {"cm" , "Compensation method ",TR_DOUBLE, false},
  {"max_iter" , "Maximum number of iterations ", TR_DOUBLE, false},
  {"abstol" , "Absolute Tolerance " , TR_DOUBLE, false},
  {"opt", "run a DC analysis up front", TR_BOOLEAN, false}
};

SVTran::SVTran() : Analysis(&ainfo, pinfo, n_par), ls_size(0),
superLU(false)
{
  paramvalue[0] = &(tf);
  paramvalue[1] = &(h);
  paramvalue[2] = &(nst = zero);
  paramvalue[3] = &(deriv = 0);
  paramvalue[4] = &(use_msv = true);
  paramvalue[5] = &(int_method = 0);
  paramvalue[6] = &(savenode = true);
  paramvalue[7] = &(permc_spec = 2);
  paramvalue[8] = &(out_steps = 100);
  paramvalue[9] = &(gcomp = 1e-4);
  paramvalue[10] = &(cm=1.0);
  paramvalue[11] = &(max_iter=2550);
  paramvalue[12] = &(abstol=0.001);
  paramvalue[13] = &(opt=false);
}

SVTran::~SVTran()
{
  // assert(!superLU);
}

void SVTran::run(Circuit* cir)
{
  nt=0;
  iter=0;
  total_iteration=0;
  scount=0;
  col_count=0;
  mnam_scount=0;
  cmflag=0;
  sparsity=0;

  if(cm)
  {
    dummy_run(cir);
    superLU=false;
    dummy_run(cir);
    cmflag=0;
  }
  else
  {
    cmflag=1;
    dummy_run(cir);
  }
}

void SVTran::dummy_run(Circuit* cir)
{
  this->cir = cir;
  char msg[80];

  superLU_print_enable = 0;
  double error = zero, ctime=zero;
  int iteration =0, flag=0, max_n_states=0,tmpidx=0;

  report(MESSAGE, "*** State Variable-Based Transient Analysis using Sparse Matrices and Newton Iterations***");
  sepLine();
  newLine();

  // Build time domain MNAM
  ElemFlag mnam_mask(LINEAR);
  TimeMNAM mnam(cir, mnam_mask);
  // Build T matrix and nonlinear element vector
  n_states = 0;
  ElemFlag mask(NONLINEAR);
  buildTIncidence(mask, cir, T, elem_vec, n_states, max_n_states);
  if(cm)
  {
    if(!cmflag)
    {
      rcomp = new double[n_states];
      for(int i=0; i <n_states; i++)
      {
				if(gcomp)
					rcomp[i]=1.0/gcomp;
      }
    }
  }
  else
  {
    rcomp = new double[n_states];
    for(int i=0; i <n_states; i++)
    {
      if(gcomp)
				rcomp[i]=1.0/gcomp;
    }
  }

  // Add the compensation resistors to the MNAM
  for (int i = 0; i < n_states; i++)
  {
    if(rcomp[i])
      mnam.setMAdmittance(T(0,i), T(1,i) , 1.0/rcomp[i]);
  }

  // Go through all the nonlinear elements
  int n_elem = elem_vec.size();
  cns.resize(n_elem);
  nss.resize(n_elem);
  n_sec_states = 0;
  for (int k=0; k < n_elem; k++)
  {
    cns[k] = elem_vec[k]->getNumberOfStates();
    nss[k] = elem_vec[k]->getNumberOfSecStates();
    n_sec_states += nss[k];
  }

  // Setup simulation variables (use circular vectors)
  ls_size = mnam.getDim();
  list_size = int((tf - nst) / h + 3);
  if (savenode)
    cU = new CircVector(list_size, ls_size);
  else
    cU = new CircVector(51, ls_size);
  if(cm)
  {
    if(!cmflag) //initialize only once
      cX = new CircVector(list_size, n_states);
		cY = new CircVector(list_size, n_sec_states);
  }
  else
  {
		cX = new CircVector(list_size, n_states);
		cY = new CircVector(list_size, n_sec_states);
  }
  cVnl = new CircVector(list_size, n_states);
  cInl = new CircVector(list_size, n_states);
  ssv = new double[ls_size+n_states];
  sf = new double[ls_size];
  rhs_vec= new double[ls_size+n_states];
  DoubleDenseColMatrix Ju(n_states , n_states);
  DoubleDenseColMatrix Ji(n_states , n_states);
  DoubleDenseColMatrix rhs_temp(ls_size, n_states);
  DoubleSparseColMatrix *M ;
  DenseIntVector myindex(ls_size,0);
  M = new DoubleSparseColMatrix(ls_size+n_states,ls_size+n_states);
  rhs_Jv_X = new double[n_states];
  elem_val = new double[(ls_size+n_states)*(ls_size+n_states)];
  row_index = new int[(ls_size+n_states)*(ls_size+n_states)];
  col_pointer = new int[ls_size+n_states+1];
  tmp_vec2 = new double[ls_size];
  tmp_vec1 = new double[ls_size];
  int myT[n_states][ls_size];
  r=new double[ls_size+n_states];
  c=new double[ls_size+n_states];

  //Set initial conditions
  LIntegMethod *l_im;
  NLIntegMethod *nl_im;
  if (int_method == 1)
  {
    l_im = new LTrapezoidal(cU, &mnam);
    nl_im = new NLTrapezoidal(cX,cY,h);
  }
  else
  {
    l_im = new LEuler(cU, &mnam);
    nl_im = new NLEuler(cX,cY, h);
  }
  DoubleSparseColMatrix M1(ls_size, ls_size);
  l_im->buildMd(M1, h);

  // Begin main loop: fixed time step by now
  report(MESSAGE, "--- Starting transient simulation ...\n");
  sepLine();
  report(MESSAGE, "| Step \t| Time (s) \t|");
  sepLine();

  for(int i=0 ; i <n_states;i ++)
    for(int j=0; j<ls_size; j++)
      myT[i][j]=0;
  for(int k =0 ; k < n_states ; k++ )	//Building T
  {
    if(T(0,k)==T(1,k))
    {
      T(0,k) =0;
      T(1,k)=0;
    }
    if (T(0,k))
    {
      tmpidx =T(0,k) -1 ;
      myT[k][tmpidx] = 1;
      myindex[tmpidx] = 1;
    }
    if(T(1,k))
    {
      tmpidx =T(1,k) -1 ;
      myT[k][tmpidx] =-1;
      myindex[tmpidx] = 1;
    }
  }

  m_i = M1.begin();   //Adding MNAM
  iend = M1.end();
  for (; m_i != iend; ++m_i)
  {
    col_pointer[col_count++] = scount;
    m_j = (*m_i).begin();
    jend = (*m_i).end();
    for (; m_j != jend; ++m_j)
    {
      elem_val[scount] = (*m_j);
      row_index[scount]=m_j.row();
      scount++;
    }

    for(int i=0;i<n_states ;i++) //Adding T
    {
      if(myT[i][col_count-1])
      {
				elem_val[scount] = myT[i][col_count-1];
				row_index[scount]=ls_size+i;
				scount++;
      }
    }
  }

  temp_mnam = new double[scount];
  memcpy(temp_mnam, elem_val, sizeof(double)*scount);
  mnam_scount = scount;

  // --- Optional DC analysis setup
  // Initial condition vectors
  DenseDoubleVector X0(n_states), Vnl0(n_states,zero);
  DenseDoubleVector Inl0(n_states,zero), Vl0(n_states,zero);
  if (opt)
  {
    // run a DC analysis
    DC * dca = new DC();
    double res = dca->getBias(cir, LINEAR, NONLINEAR,
		X0, Vnl0, Inl0, Vl0);
    sprintf(msg, "DC analysis residual = %g", res);
    report(MESSAGE, msg);

    // Initialize variables related to DC simulation
    assert(int(X0.size()) == n_states);

    // Initialize the circular vectors correctly
    for (int j = 0; j < list_size; j++)
    {
      cX->getCurrent() = X0;
      cX->advance();
      cVnl->getCurrent() = Vnl0;
      cVnl->advance();
      cInl->getCurrent() = Inl0;
      cInl->advance();
      for (int i = 0; i < n_states; i++)
        cU->getCurrent()[i] = Vl0[i];
      cU->advance();
    }
    delete dca;
  }

  n_tsteps = int(tf / h + 1);
  tdsv = new TimeDomainSV(&(cX->getCurrent()[0]),
	&(cVnl->getCurrent()[0]),
	&(cInl->getCurrent()[0]), max_n_states, nl_im);

  for ( nt=1; nt < n_tsteps; nt++)
  {
    ctime = h * nt;
    iteration = 0;
    tdsv->setTime(&(cX->getCurrent()[0]),&(cVnl->getCurrent()[0]),
		&(cInl->getCurrent()[0]),nt, ctime, h);//first_time=true

    DenseDoubleVector& Inl = cInl->getCurrent();
    DenseDoubleVector& Xnl = cX->getCurrent();
    DenseDoubleVector& Vnl = cVnl->getCurrent();
    DenseDoubleVector& Ul  = cU->getCurrent();
    memset( sf,0,sizeof(double)*ls_size);
    // Update compressed source vector
    l_im->buildSf(sf, ctime);

    do  //Start Newton Iterations
    {
			memcpy(ssv,sf,sizeof(double)*ls_size);
			memset(elem_val,0,sizeof(double)*((ls_size+n_states)*(ls_size+n_states)));
			memcpy(elem_val, temp_mnam, sizeof(double)*mnam_scount);
			scount = mnam_scount;
			col_count = ls_size;
			mtl::set_value(rhs_temp, 0);
			int ibase =0 ,jbase=0;
			memset(rhs_Jv_X, 0 , n_states*sizeof(double));
			int n_elem = elem_vec.size();
			int prev_scount=0;
			for (int  k = 0; k < n_elem; k++)
			{
				int c_cns = elem_vec[k]->getNumberOfStates();
				// Set base index in interface object
				tdsv->setBase(ibase, cns[k],jbase,nss[k]);
				elem_vec[k]->svTran(tdsv);
				elem_vec[k]->deriv_svTran(tdsv);        // Call element evaluation

				DoubleDenseColMatrix& Ju_elem = tdsv->getJu();
				DoubleDenseColMatrix& Ji_elem = tdsv->getJi();

				//DO NOT TOUCH WILL TRY TO MAKE IT EFFICIENT
				for(int m=0 ; m<c_cns; m++) //Ji col counter
				{
					col_pointer[col_count++]= scount;
					prev_scount = scount;
					for (int j=0; j < c_cns; j++) //T' row counter
					{
						tmpidx=0;
						if(T(0,j+ibase)&&Ji_elem(j,m)) //multiply with 1
						{
							tmpidx = T(0,j+ibase)-1;
							int i =0;
							for( i = prev_scount; i< scount;i++)
							{
								if(row_index[i]==tmpidx) //element already exists
								{
									elem_val[i]+=Ji_elem(j,m);
									break;
								}
								if(row_index[i]>tmpidx)
								{
									for(int ti = scount ; ti > i ; ti--)
									{
										elem_val[ti]= elem_val[ti-1];
										row_index[ti]=row_index[ti-1];
									}
									elem_val[i]= Ji_elem(j,m);
									row_index[i]=tmpidx;
									scount++;
									break;
								}
							}
							if(i==scount) //add new elem at position i
							{
								elem_val[i]= Ji_elem(j,m);
								row_index[i]=tmpidx;
								scount++;
							}
						}
						if(T(1,j+ibase)&&Ji_elem(j,m)) //multiply with -1
						{
							tmpidx = T(1,j+ibase)-1;
							int i =0;
							for( i = prev_scount; i< scount;i++)
							{
								if(row_index[i]==tmpidx) //element already exists
								{
									elem_val[i]-=Ji_elem(j,m);
									break;
								}
								if(row_index[i]>tmpidx)
								{
									for(int ti = scount ; ti > i ; ti--)
									{
										elem_val[ti]= elem_val[ti-1];
										row_index[ti]=row_index[ti-1];
									}
									elem_val[i]= -Ji_elem(j,m);
									row_index[i]=tmpidx;
									scount++;
									break;
								}
							}
							if(i==scount) //add new elem at poistion i
							{
								elem_val[i]= -Ji_elem(j,m);
								row_index[i]=tmpidx;
								scount++;
							}
						}

						//rhs_temp(i,m+ibase)= temp;
					}
					for(int i=prev_scount; i <scount;i++)
						rhs_temp(row_index[i],m+ibase)=elem_val[i];

					for(int i=0; i < c_cns; i++)
					{
						elem_val[scount]= -Ju_elem(i,m);
						row_index[scount]=ls_size+i+ibase;
						scount++;
						rhs_Jv_X[i+ibase]+=Ju_elem(i,m)*Xnl[m+ibase];
					}
				}
				//END OF DO NOT TOUCH
				ibase += cns[k];
				jbase += nss[k];
			}

			tdsv->clearflag();
			//Multiplying T'JiXi
			multiply(rhs_temp , Xnl, ssv);
			for(int k=0 ; k <n_states; k++) //Multiplying T'*Inl
			{
				if(T(0,k))
				{
					tmpidx=(T(0,k)) -1;
					if(rcomp[k])
						ssv[tmpidx] =ssv[tmpidx]-(Inl[k]-Vnl[k]/rcomp[k]);
					else
						ssv[tmpidx] = ssv[tmpidx]-(Inl[k]);
				}
				if(T(1,k) )
				{
					tmpidx =(T(1,k)) -1;
					if(rcomp[k])
						ssv[tmpidx] =ssv[tmpidx]+(Inl[k]-Vnl[k]/rcomp[k]);
					else
						ssv[tmpidx]=ssv[tmpidx]+(Inl[k]);
				}
			}

			for(int i = 0 ; i < n_states ; i++)
				ssv[ls_size+i] =Vnl[i]- rhs_Jv_X[i];

			rhs_p =ssv;
			//report(MESSAGE,"Printing LHS...\n");
      /*for(int i= 0; i < scount; i++)*/
      //{
        //sprintf(msg, "M[%d]= %f, %d ", i , elem_val[i],row_index[i]);
        //report(MESSAGE,msg);
      /*}*/
			//  sepLine();
			
			//report(MESSAGE,"Printing RHS...\n");
			/*for(int i= 0; i < ls_size+n_states; i++)*/
			//{
				//sprintf(msg, "RHS[%d]= %f  ", i , ssv[i]);
				//report(MESSAGE,msg);
			/*}*/

			superLUFactor(M,ssv);

			/*
			for(int i=0;i<ls_size+n_states;i++)
				cout<<"ssv["<<i<<"] = "<<ssv[i]<<endl;
			if(iteration==5)
				exit(1);
			sepLine();
			*/
			error = zero;
			flag =0;
			for(int j= 0 ; j < ls_size && flag==0 ; j++)
			{
				error = abs(Ul[j] - ssv[j] );
				if(error > abstol)
					flag = 1;
			}

			for(int j =0 ; j < n_states && flag==0; j++)
			{
				error = abs(Xnl[j] - ssv[ls_size+j] );
				if(error > abstol)
					flag = 1; //continue
			}

			if(!cmflag)
			{
				iteration = max_iter;
				nt=n_tsteps;
				flag=1;
			}
			iteration++;
			updateVInl(&(ssv[0]));
			if(nt==1&&flag==1&&iteration<max_iter)
				freeSuperLU();

    } while(iteration < max_iter && flag ==1 );

    total_iteration =total_iteration +iteration;
    if(iteration >= max_iter)
    {
      sprintf(msg , "Newton Raphson  didn't converge....");
      report(MESSAGE, msg);
    }
    int jbase =0,ibase=0;
    int n_elem = elem_vec.size();
    for (int k = 0; k < n_elem; k++)
    {
      // int c_cns = elem_vec[k]->getNumberOfStates();
      // Set base index in interface object
      tdsv->setBase(ibase, cns[k],jbase,nss[k]); // Call element evaluation
      elem_vec[k]->svTran(tdsv);
      if(!cmflag)
      {
        elem_vec[k]->nx=0;
        elem_vec[k]->ndx=0;
      }
      ibase += cns[k];
      jbase+= nss[k];
    }
    // DO NOT TOUCH TOUCH
    if(cm && (!cmflag)) //Find rcomp
    {
      DenseDoubleVector tmp_vec4(ls_size,0);
      DenseDoubleVector linear_i(ls_size,0);
      DenseDoubleVector delta_i(ls_size,0);
      //multiply T' and Inl
      for(int k=0 ; k <n_states; k++)
      {
        rcomp[k]=0;
        if(T(0,k))
        {
          int tmpidx=(T(0,k)) -1 ;
          tmp_vec4[tmpidx] =tmp_vec4[tmpidx]+cInl->getCurrent()[k];
        }
				if(T(1,k) )
				{
					int tmpidx =(T(1,k)) -1;
					tmp_vec4[tmpidx] =tmp_vec4[tmpidx]-cInl->getCurrent()[k];
				}
      }

      DenseDoubleVector icomp(n_states,0);
      for(int i=0; i <ls_size; i++)
      {
        //go row by row to compensate delta
        if(myindex[i])
        {
          double denom =0;
					//multiply M1 and U
					for(int j =0; j <ls_size;j++)
					{
						double tx = M1(i,j);
						linear_i[i] +=tx * cU->getCurrent()[j];
					}
					delta_i[i]= linear_i[i]+tmp_vec4[i];
					DenseIntVector sum_index(n_states,0);
					for(int k=0; k <n_states ;k++)
					{ //check which currents are involved
						double tx = myT[k][i];
						if(tx && !icomp[k])
						{
							denom += tx*cInl->getCurrent()[k];
							sum_index[k]++;
						}
					}
					for(int k=0; k<n_states; k++)
					{
						if(sum_index[k])
						{
							if((delta_i[i]==0)||(cInl->getCurrent()[k]==0))
								rcomp[k]=0;
							else
								rcomp[k]=(cVnl->getCurrent()[k]*denom)/(delta_i[i]*cInl->getCurrent()[k]);
							rcomp[k]= abs(rcomp[k]);
							if(rcomp[k] && (rcomp[k]<100))
								rcomp[k]=100;
							icomp[k]=1;
						}
					}
				}
      }
    } //end of compensation calculations
    //END OF DO NOT TOUCH

    // Update previous derivatives in integration method (if required)
    if(nt>0)
    {
      nl_im->store();
      l_im->store();
    }
    delete tdsv;

    // Advance the circular lists for the next step.
    // After dummy run ,run is started with the values of cX from dummy run
    // as Adolc does not erase the tape.
    if(cmflag)
    {
      cU->advance();
      cX->advance();
      cY->advance();
      cVnl->advance();
      cInl->advance();
    }

    //set initial cX at next time step = calculated cX at this time step
    for(int i =0 ; i < n_states ; i++)
      cX->getCurrent()[i] =ssv[ls_size+i];

    if(nt>0)    //transient constructor
    {
      tdsv = new TimeDomainSV(&(cX->getCurrent()[0]),&(cVnl->getCurrent()[0]),
			&(cInl->getCurrent()[0]),max_n_states,nl_im);
      tdsv->clearflag();
    }
    else
    {          //dc constructor
      tdsv = new TimeDomainSV(&(cX->getCurrent()[0]),
			&(cVnl->getCurrent()[0]),
			&(cInl->getCurrent()[0]),max_n_states);
      tdsv->clearflag();
    }

    // Print table line
    if (!(nt % out_steps))
    {
      sprintf(msg, "| %d  \t| %g  \t|", nt, ctime);
      report(MESSAGE, msg);
    }
  } // end time loop
  freeSuperLU();
  superLU = true;
  if(cmflag)
    doOutput();
  cmflag=1;
  delete nl_im;
  delete l_im;
  delete cInl;
  delete cVnl;
  //delete cX;
  delete cU;
  return;
}

void SVTran::superLUFactor(DoubleSparseColMatrix*& M, double* ssv)
{
  // Call freeSuperLU to release memory
  // assert(!superLU);
  int info,nrhs=1;
  iter++;
  double *rhs_p = &(ssv[0]);
  col_pointer[col_count] = scount;
  fact      = 'E';
  equed     = 'B';
  trans     = 'N';
  refact    = 'N';
  // double recip_pivot_growth;
  // double rcond;
  // These are vectors of 1 element (nrhs elements)
  // double ferr(0), berr(0);
  // mem_usage_t mem_usage;
  // factor_param_t *ftp = NULL;
  panel_size=10;
  relax=5;

  if(nt==1)
  {
    dCreate_CompCol_Matrix(&A, n_states+ls_size, n_states+ls_size, scount,
		elem_val, // elem_val pointer
		row_index, // row_index pointer
		col_pointer, // col_pointer pointer
		NC, _D, GE);
    // dCreate_Dense_Matrix(&B, n_states+ls_size, nrhs, rhs_p, n_states+ls_size, DN, _D, GE);

    dCreate_Dense_Matrix(&X, n_states+ls_size, nrhs, rhs_p, 
                         n_states+ls_size, DN, _D, GE);
    StatInit(panel_size, relax);
    perm_r = new int[n_states+ls_size];
    perm_c = new int[n_states+ls_size];
    etree = new int[ls_size+n_states];
    get_perm_c(permc_spec, &A, perm_c);
    sp_preorder(&refact,&A,perm_c,etree,&AC);
    dgstrf(&refact, &AC, 1.0, 0.0, relax, panel_size, etree,
           NULL, 0, perm_r, perm_c, &L, &U, &info);
    dgstrs(&trans,&L,&U,perm_r,perm_c,&X,&info);
  }
  else
  {
    refact='Y';
    dgstrf(&refact,&AC,1.0,0.0, relax,panel_size,etree,NULL,0,perm_r,perm_c,&L,&U,&info);
    dgstrs(&trans,&L,&U,perm_r,perm_c,&X,&info);
  }


  /*
	memset(c, 0, (ls_size+n_states)*sizeof(double));
	memset(r, 0, (ls_size+n_states)*sizeof(double));

	dCreate_CompCol_Matrix(&A, n_states+ls_size, n_states+ls_size, scount,
	elem_val, // elem_val pointer
	row_index, // row_index pointer
	col_pointer, // col_pointer pointer
	NC, _D, GE);
	dCreate_Dense_Matrix(&B, n_states+ls_size, nrhs, rhs_p, n_states+ls_size, DN, _D, GE);
	dCreate_Dense_Matrix(&X, n_states+ls_size, nrhs, tmp_p, n_states+ls_size, DN, _D, GE);
	perm_r = new int[n_states+ls_size];
	perm_c = new int[n_states+ls_size];
	get_perm_c(permc_spec, &A, perm_c);
	etree = new int[ls_size+n_states];

	dgssvx(&fact, &trans, &refact, &A, ftp, perm_c, perm_r, etree, &equed, r, c,
	&L, &U, work, lwork, &B, &X, &recip_pivot_growth, &rcond,
	&ferr, &berr, &mem_usage, &info);
	*/
	if (info)
	{
		report(FATAL, "There was a problem factoring the MNAM.");
		report(MESSAGE, "Try increasing tstep..");
	}

  // NCformat *Ustore;
  //SCformat *Lstore;
  //Lstore = (SCformat *) L.Store;
  //Ustore = (NCformat *) U.Store;

  fflush(stdout);
  superLU = true;
  sparsity =(double) scount/((n_states+ls_size)*(n_states+ls_size));
  sparsity = sparsity*100;
  scount=0;
  col_count =0;

}

void SVTran::freeSuperLU()
{
  assert(superLU);
  // Free vectors
  delete [] etree;
  delete [] perm_c;
  delete [] perm_r;
  // Free supermatrix structures
  Destroy_SuperMatrix_Store(&A);
  Destroy_SuperMatrix_Store(&X);
  // Free L and U matrices
  if ( lwork >= 0 )
  {
    Destroy_SuperNode_Matrix(&L);
    Destroy_CompCol_Matrix(&U);
  }
  // delete [] col_pointer;
  // delete [] row_index;
  // delete [] elem_val;
  superLU = false;
}

void SVTran::multiply(DoubleDenseColMatrix& a , DenseDoubleVector& b ,
double* c)
{
  int imax = a.nrows();
  int jmax = a.ncols();

  for(int i =0 ; i < imax ; i++)
  {
    for(int j=0; j < jmax; j++)
      c[i] += a(i,j) * b[j];
  }
}


void SVTran::updateVInl(double* x_p)
{
  memcpy(&(cU->getCurrent()[0]),x_p,sizeof(double)*ls_size);
  memcpy(&(cX->getCurrent()[0]),&(x_p[ls_size]),sizeof(double)*n_states);
}

void SVTran::doOutput()
{
  // First check if the result matrices contain any data
  assert(cX);
  cout<<"Number of state variables :"<<n_states<<endl;
  cout<<"Size of sparse matrix :"<<n_states+ls_size<<endl;
  cout<<"Sparsity of the matrix :"<<sparsity<<"%"<<endl;
  cout<<"Total Number of Newton Iterations :"<<total_iteration<<endl;

  report(MESSAGE, "--- Writing output vectors ...");

  int out_size = list_size - 2;
  // Allocate and copy global time vector
  allocTimeV_P(out_size);
  for (int tindex = 0; tindex < out_size; tindex++)
    TimeV_P[tindex] = nst + h * tindex;

  // Temporary vectors
  DenseDoubleVector tmp_x(out_size);
  DenseDoubleVector tmp_i(out_size);
  DenseDoubleVector tmp_u(out_size);

  // Now fill currents and port voltages of time domain devices.
  int current_ns;
  int n_elem = elem_vec.size();   // Number of elements
  int i=0;
  for (int k=0; k < n_elem; k++)
  {
    current_ns = elem_vec[k]->getNumberOfStates();
    for(int j=0; j< current_ns; j++)
    {
      // For the current, decompensate while copying.
      for (int tindex=0; tindex < out_size; tindex++)
      {
				tmp_x[tindex] = cX->getPrevious(out_size - tindex)[j+i];
				tmp_i[tindex] = cInl->getPrevious(out_size - tindex)[j+i];
				tmp_u[tindex] = cVnl->getPrevious(out_size - tindex)[j+i];
      }
      elem_vec[k]->getElemData()->setRealX(j, tmp_i);
      elem_vec[k]->getElemData()->setRealI(j, tmp_i);
      elem_vec[k]->getElemData()->setRealU(j, tmp_u);
    }
    i += current_ns;
  }

  if (savenode)
  {
    // For each terminal, assign voltage vector
    Terminal* term = NULL;
    cir->setFirstTerminal();
    while((term = cir->nextTerminal()))
    {
      // Get MNAM index
      if (term->getRC())
      {
				// Remember that MNAM indices begin at 1
				i = term->getRC() - 1;
				for (int tindex=0; tindex < out_size; tindex++)
					tmp_u[tindex] = cU->getPrevious(out_size - tindex)[i];
      }
      else
				// This is a reference terminal
			mtl::set_value(tmp_u, zero);

      // Set terminal vector
      term->getTermData()->setRealV(tmp_u);
    }
  }
}

