// Class definition for the new general state variable transient analysis
//
// Author:
//          Sonali R. Luniya
//

#ifndef SVTran_h
#define SVTran_h 1

#include "TimeMNAM.h"
#include "TimeDomainSV.h"
#include "NLSInterface.h"
#include "Euler.h"
#include "Trapezoidal.h"
#include "DC.h"
#include "Analysis.h"
#include "CircVector.h"
#include "../freeda_mtl.h"

extern "C"
{
#undef _S
#undef _C
#include "dsp_defs.h"
#include "util.h"
}

// Main class definition follows
class SVTran: public Analysis
{
	public:

  SVTran();

  ~SVTran();

  // (from Analysis)
  // The main analysis routine.
  virtual void run(Circuit* cir);
  virtual void dummy_run(Circuit* cir);

  //Return the name of the element
  static const char* getNetlistName()
  {
     return ainfo.name;
  }

  // Write output vectors
  void doOutput();

	private:

  // void superLUFactor (SparseMatrix *& M, DenseVector& ssv,
	// double& mcond, double& minfo);
  void superLUFactor(DoubleSparseColMatrix *& M, double* ssv);
  void freeSuperLU();
  void updateVInl(double* x_p);
  void multiply(DoubleDenseColMatrix& a , DenseDoubleVector& b, double* c);
  // void multiply(DoubleMatrix& a , ExtVector& b , DoubleVector& c);
  Circuit* cir;
  int n_states,n_sec_states;
  IntDenseColMatrix T;
  ElementVector elem_vec;
  DenseIntVector cns, nss;    // Vectors for number of primary and secondary SV.
  TimeDomainSV *tdsv;
  int ls_size;
  int n_tsteps;
  int list_size;
  CircVector *cU;
  CircVector *cX;
  CircVector *cY;
  CircVector *cVnl;
  CircVector *cInl;
  CircVector *cTime;
  //CircVector *cTimestep;

  DoubleDenseRowMatrix Msv;
  DoubleDenseColMatrix rhs_temp;
  DenseDoubleVector c_v,r_v;
  // SuperLU variables
  SuperMatrix A, L, U, B, X,AC;
  int *perm_r;
  int *perm_c;
  int *etree;
  int panel_size,relax,firsttime;
  char fact, equed, trans, refact;
  double *r, *c;
  double *rhs_p, *tmp_p,*ju_p,*ji_p;
  void *work;
  int lwork;
  bool superLU;
  double *elem_val, *rcomp;
  double  *rhs_Jv_X, *tmp_vec1, *tmp_vec2, *rhs_vec, *sf, *ssv, *temp_mnam;
  // double* Integ_predX;
  // double* Integ_predU;
  int* row_index;
  int* col_pointer;
  DoubleSparseColMatrix::iterator m_i, iend;
  DoubleSparseColMatrix::OneD::iterator m_j, jend;
  // ------------------- Parameter-related variables
  // Analysis information
  static ItemInfo ainfo;
  // Number of parameters of this element
  static const unsigned n_par;

  // Analysis parameters
  double h, tf, nst, gcomp,cm;
  int deriv, int_method, permc_spec, out_steps,max_iter;
  bool use_msv, savenode, opt;

  double sparsity,abstol;
  int nt,iter,total_iteration, scount, col_count, mnam_scount,cmflag;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif

