// Class definition for the new general state variable transient analysis
// Author: Carlos E. Christoffersen

#ifndef SVTran2_h
#define SVTran2_h 1

#include <fstream>
#include <iomanip>
#include "Analysis.h"
#include "OFunction.h"
#include "CircVector.h"
#include "TimeMNAM.h"
#include "TimeDomainSV.h"
#include "NLSInterface.h"
#include "Euler.h"
#include "Trapezoidal.h"
#include "DC.h"

extern "C"
{
#undef _S
#undef _C
#include "dsp_defs.h"
#include "util.h"
}

// Main class definition follows
class SVTran2 : public Analysis, public OFunction
{
public:
  SVTran2();
  ~SVTran2();

  // a function to return the simulation time step
  inline double & getStep()
  {
    return h;
  }

  // (from Analysis)
  // The main analysis routine.
  virtual void run(Circuit* cir);

  //Return the name of the element
  static const char* getNetlistName()
  {
    return ainfo.name;
  }

  // (from OFunction)
  // Evaluate error function vector F given X
  virtual void func_ev(double* X, double* F);
  // Evaluate Jacobian
  virtual void jacobian(double* X, DoubleDenseExtColMatrix& J);

private:
  void superLUFactor(DoubleSparseColMatrix& M);
  void freeSuperLU();
  void f1(DenseDoubleVector& s1, DenseDoubleVector& v1l);
  void updateVInl(double* x_p);
  void updateU(DenseDoubleVector& s1);
  void buildMsv();

  Circuit* cir;
  int n_states, n_sec_states;
  IntDenseColMatrix T;
  ElementVector elem_vec;
  // Vectors for number of primary and secondary SV.
  DenseIntVector cns, nss;
  TimeDomainSV *tdsv;
  int ls_size;
  int n_tsteps;
  int list_size;
  DenseDoubleVector ssv, vl1, itmp2, tmp_v, rhs_v;
  CircVector *cU;
  CircVector *cX;
  CircVector *cY;
  CircVector *cVnl;
  CircVector *cInl;
  DoubleDenseRowMatrix Msv;

  // SuperLU variables
  SuperMatrix A, L, U, B;
  int *perm_r;
  int *perm_c;
  double *rhs_p;
  bool superLU;
  double* elem_val;
  int* row_index;
  int* col_pointer;

  // ------------------- Parameter-related variables
  // Analysis information
  static ItemInfo ainfo;
  // Number of parameters of this element
  static const unsigned n_par;
  // Analysis parameters
  double h, tf, nst, gcomp;
  int deriv, int_method, permc_spec, out_steps;
  bool use_msv, opt;
  // Parameter information
  static ParmInfo pinfo[];
};

#endif

