// Class definition for the new general state variable transient analysis
//
// Author:
//          Sonali R. Luniya

#ifndef SVTran3_h
#define SVTran3_h 1

#include "Analysis.h"
#include "CircVector.h"
#include "NLSInterface.h"
#include "Euler.h"
#include "Trapezoidal.h"
#include "../freeda_mtl.h"

extern "C"
{
	#undef _S
	#undef _C
	#include "dsp_defs.h"
	#include "util.h"
}

// Main class definition follows
class SVTran3 : public Analysis
{
	public:

  SVTran3();

  ~SVTran3();

  // (from Analysis)
  // The main analysis routine.
  virtual void run(Circuit* cir);
  virtual void dummy_run(Circuit* cir);
  
  //Return the name of the element
  static const char* getNetlistName()
  {
     return ainfo.name;
  }

  // Write output vectors
  void doOutput();
  double calc_tol(double timestep);
	private:

  // void superLUFactor(SparseMatrix *& M, DenseVector& ssv, double& mcond, double& minfo);
  void superLUFactor(DoubleSparseColMatrix *& M,double*  ssv);
  void freeSuperLU();
  void updateVInl(double* x_p);
  void multiply(DoubleDenseColMatrix& a , DenseDoubleVector& b, double* c);
  void PredictNewStep(double* ssv, double *timestep);
  Circuit* cir;
  int n_states,n_sec_states;
  IntDenseColMatrix T;
  ElementVector elem_vec;
  DenseIntVector cns, nss;    // Vectors for number of primary and secondary SV.
  TimeDomainSV *tdsv;
  LIntegMethod *l_im;
  NLIntegMethod *nl_im;
  int ls_size;
  int n_tsteps;
  int list_size;
  DenseDoubleVector  vl1, itmp2, tmp_v, rhs_v, c_v, r_v;
  CircVector *cU;
  //CircVector *cX; //Have this only if the *cX is not a global array.
  CircVector *cY;
  CircVector *cVnl;
  CircVector *cInl;
  CircVector *cTime;
  CircVector *cTimestep;

  DoubleDenseRowMatrix Msv;

  // SuperLU variables
  SuperMatrix A, L, U, B, X,AC;
  int *perm_r;
  int *perm_c;
  int *etree;
  int panel_size,relax,firsttime;
  char fact, equed, trans, refact;
  double *r, *c;
  double *rhs_p, *rcomp, *ssv, *sf, *rhs_vec;
  void *work;
  int lwork;
  bool superLU;
  double* elem_val;
  double* rhs_Jv_X;
  double* Integ_predX;
  double* Integ_predU;
  double* Integ_predVnl;
  int* row_index;
  int* col_pointer;
  DoubleSparseColMatrix::iterator m_i, iend;
  DoubleSparseColMatrix::OneD::iterator m_j, jend;
  // ------------------- Parameter-related variables
  // Analysis information
  static ItemInfo ainfo;
  // Number of parameters of this element
  static const unsigned n_par;
  // Analysis parameters
  double h, tf, nst, gcomp,cm,abstol,reltol,tol,tmin;
  int deriv, int_method, permc_spec, out_steps,max_iter;
  bool use_msv, savenode;

  CircVector *cX;
  int nt ,total_iteration, scount , col_count, mnam_scount,cmflag,cktorder,changestep;
  double prev_timestep, new_timestep, ptime,ctime, net_timestep, sparsity;

  // Parameter information
  static ParmInfo pinfo[];
};

#endif

