#include "Trapezoidal.h"

NLTrapezoidal::NLTrapezoidal(CircVector*& cX, CircVector*& cY,
CircVector*& cTime, double& h)
{
  this->cX = cX;
  this->cY = cY;
  this->cTime=cTime;
  a = 2. / h;
  a2 = a * a;
  da = 2. * a;
  xpn = DenseDoubleVector(cX->getCurrent().size(), zero);
  xsn = DenseDoubleVector(cX->getCurrent().size(), zero);
  ypn = DenseDoubleVector(cY->getCurrent().size(), zero);
}

NLTrapezoidal::NLTrapezoidal(CircVector*& cX, CircVector*& cY, CircVector*& cU,
CircVector*& cTime, double& h)
{
  this->cX = cX;
  this->cY = cY;
  this->cU = cU;
  this->cTime=cTime;
  a = 2. / h;
  a2 = a * a;
  da = 2. * a;
  xpn = DenseDoubleVector(cX->getCurrent().size(), zero);
  xsn = DenseDoubleVector(cX->getCurrent().size(), zero);
  ypn = DenseDoubleVector(cY->getCurrent().size(), zero);
}

NLTrapezoidal::NLTrapezoidal(CircVector*& cX, CircVector*& cY, double& h)
{
  this->cX = cX;
  this->cY = cY;
  this->cTime=NULL;
  a = 2. / h;
  a2 = a * a;
  da = 2. * a;
  xpn = DenseDoubleVector(cX->getCurrent().size(), zero);
  xsn = DenseDoubleVector(cX->getCurrent().size(), zero);
  ypn = DenseDoubleVector(cY->getCurrent().size(), zero);
}

double NLTrapezoidal::derivX(const int& index)
{
  return (cX->getCurrent()[index] - cX->getPrevious(1)[index]) * a
	- xpn[index];
}

double NLTrapezoidal::deriv2X(const int& index)
{
  return (cX->getCurrent()[index] - cX->getPrevious(1)[index]) * a2
	- da * xpn[index] - xsn[index];
}

double NLTrapezoidal::getdx_dtFactor()
{
  return a;
}

double NLTrapezoidal::getd2x_dt2Factor()
{
  return a2;
}

//this is the new delay routine for the Non linear trapezoidal technique
double NLTrapezoidal::delayX(const int& index, const double& t)
{
  // first we need to find the where in the circular matrix the state variable
  // required is stored
  // double time_delay
  if(cTime==NULL)
	{
		double delta_step = t * a / 2.;
		int time_idx = int(delta_step);
		delta_step -= double(time_idx);
		const double& v2 = cX->getPrevious(time_idx)[index];
		const double& v1 = cX->getPrevious(time_idx + 1)[index];
		// Interpolate between v1 and v2
		return v2 + (v1 - v2) * delta_step;
	}
  else
	{
		double time_delay = cTime-> getCurrent()[0];
		int current_pos = cTime->curr_pos;
		double rtime = time_delay-t;
		//define some temporary variables
		double temp ;
		int time_idx;
		int  i=1;
		do
		{
			temp=cTime->getPrevious(i)[0];
			if(time_delay-temp<t)
				// if(temp-time_delay>0)
	    // prev_temp=temp;
	    i++;
		}
		while(time_delay-temp<t && (i<=current_pos));
		// while(temp-time_delay>0);
		time_idx = i;
		double ttime2 = cTime->getPrevious(time_idx)[0];
		double ttime1 = cTime->getPrevious(time_idx+1)[0];
		double delta_step= abs(rtime-ttime2);
		delta_step =(delta_step)/(ttime2-ttime1);
		// double delta_step=t/(temp-prev_temp);
		//delta_step -=int(delta_step);
		const double& v2 = cX->getPrevious(time_idx)[index];
		const double& v1 = cX->getPrevious(time_idx + 1)[index];
		// Interpolate between v1 and v2
		return v2 + (v1 - v2) * delta_step;
	}
}

double NLTrapezoidal::derivY(const int& index, const double& currval)
{
  // Save Current value
  cY->getCurrent()[index] = currval;
  return (currval - cY->getPrevious(1)[index]) * a - ypn[index];
}


double NLTrapezoidal::getDelayXFactor(const double& t)
{
  if(cTime==NULL)
	{
		// First, find where to interpolate
		double delta_step = t * a / 2.;
		int time_idx = int(delta_step);
		delta_step -= double(time_idx);
		if (time_idx)
			return zero;
		else
			return one - delta_step;
	}
  else
	{
		// First, find where to interpolate
		double time_delay= cTime->getCurrent()[0] ;
		//define some temporary variables
		double prev_temp, temp ;
		int time_idx;
		int  i=1;
		do
		{
			temp=cTime->getPrevious(i)[0];
			if(temp-time_delay>0)
				prev_temp=temp;
			i++;
		}
		while(temp-time_delay>0);
		time_idx= i;
		double delta_step=t/(temp-prev_temp);
		delta_step -= int(delta_step);
		if (time_idx)
			return zero;
		else
			return one- delta_step;
	}
}

void NLTrapezoidal::changeStep(const double& h)
{
  a = 2. / h;
  a2 = a * a;
  da = 2. * a;
}

void NLTrapezoidal::getNSamples(int &nx, int &ndx)
{
  nx = 1;
  ndx = 1;
}

double NLTrapezoidal::deriv(double* x, double* dx)
{
  return a * (x[0] - x[1]) - dx[1];
}

void NLTrapezoidal::predictX(int n_states, double *Integ_predX,
CircVector*& cTimestep)
{
  for(int i=0; i <n_states; i++)
	{
		double temp= (cX->getPrevious(1)[i] - cX->getPrevious(2)[i]);
		temp = temp/cTimestep->getPrevious(1)[0];
		Integ_predX[i] = cX->getCurrent()[i] = temp * cTimestep->getCurrent()[0]
		+ cX->getPrevious(1)[i];
	}
}

void NLTrapezoidal::predictU(int ls_size, double *Integ_predU,
CircVector*& cTimestep)
{
  for(int i=0; i <ls_size; i++)
	{
		double temp= (cU->getPrevious(1)[i] - cU->getPrevious(2)[i]);
		temp = temp/cTimestep->getPrevious(1)[0];
		Integ_predU[i] = cU->getCurrent()[i] = temp * cTimestep->getCurrent()[0]
		+ cU->getPrevious(1)[i];
	}
}

void NLTrapezoidal::store()
{
  // Update previous derivative vectors
  int iend = xpn.size();
  for (int i=0; i < iend; i++)
  {
    xsn[i] = (cX->getCurrent()[i] - cX->getPrevious(1)[i]) * a2
		- da * xpn[i] - xsn[i];
    xpn[i] = a * (cX->getCurrent()[i] - cX->getPrevious(1)[i]) - xpn[i];
  }
  int jend =ypn.size();
	for (int i=0; i < jend; i++)
	{
		ypn[i] = a * (cY->getCurrent()[i] - cY->getPrevious(1)[i]) - ypn[i];
	}
	return;
}

LTrapezoidal::LTrapezoidal(CircVector*& cU, TimeMNAM* mnam)
{
  this->cU = cU;
  this->mnam = mnam;
  size = mnam->getDim();
  s2 = DenseDoubleVector(size);
  upn = DenseDoubleVector(size, zero);
  tmp1v = DenseDoubleVector(size);
  a = zero;
}

void LTrapezoidal::buildMd(DoubleSparseColMatrix& M, const double& h)
{
  a = 2. / h;
  mnam->getMatrices(M1, M1p);
  mtl::copy(M1, M);
  mtl::add(scaled(M1p, a), M);
}

void LTrapezoidal::buildSf(DenseDoubleVector& s1, const double& ctime)
{
  assert(a);
  mnam->getSource(ctime, s2);
  ExtDoubleVector u_n(&(cU->getPrevious(1)[0]), size);
  mtl::add(upn, scaled(u_n, a), tmp1v);
  mtl::mult(M1p, tmp1v, s2, s1);
}

void LTrapezoidal::buildSf(double* s1, const double& ctime)
{
  assert(a);
  mnam->getSource(ctime, s2);
  memcpy(s1,&(s2[0]),sizeof(double)*size);
  DenseDoubleVector& u_n = cU->getPrevious(1);
  multiply(u_n, s1);
}

void LTrapezoidal::multiply(DenseDoubleVector& u_n, double* s1)
{
  col_count=0,row_index=0;
  temp=0;
  Sllist tmp_ptr=NULL, prev_ptr=NULL;
  tmp_ptr = mnam->Lhead;
  prev_ptr=tmp_ptr;

	while(tmp_ptr)
	{
    if(tmp_ptr->row==prev_ptr->row)
		{
      col_count=tmp_ptr->col;
			temp += tmp_ptr->val *( u_n[col_count] * a+upn[col_count]);
			prev_ptr=tmp_ptr;
			tmp_ptr=tmp_ptr->next;
    }
    else
		{
      s1[prev_ptr->row]+=temp;
      prev_ptr=prev_ptr->next;
      temp=0;
    }
  }
  s1[prev_ptr->row]+=temp;
}

void LTrapezoidal::store()
{
  // Update previous derivative vector
  for (int i=0; i < size; i++)
	{
    upn[i] = a * (cU->getCurrent()[i] - cU->getPrevious(1)[i]) - upn[i];
  }
  return;
}

