#include "../../../../network/ElementManager.h"
#include "../../../../network/AdolcElement.h"
#include "AbmVCann.h"

// Static members
const unsigned AbmVCann::n_par = 4;

// Element information
ItemInfo AbmVCann::einfo = 
{
  "abmvcann",
  "Cann Behavorial Model",
  "Minsheng Li",
  DEFAULT_ADDRESS"category:behavioral",
  "2005_04_10"
};

// Parameter information
ParmInfo AbmVCann::pinfo[] = 
{
  {"g", "Element parameter", TR_DOUBLE, true},
  {"l", "Element parameter", TR_DOUBLE, true},
  {"s", "Element parameter", TR_DOUBLE, true},
  {"rout", "Element parameter", TR_DOUBLE, true}
};


AbmVCann::AbmVCann(const string& iname) : AdolcElement(&einfo, pinfo, n_par, iname)
{
  // Set default parameter values
  paramvalue[0] = &g;
  paramvalue[1] = &l;
  paramvalue[2] = &s;
  paramvalue[3] = &rout;

  // Set the number of terminals
  setNumTerms(3);

  // Set flags
  setFlags(NONLINEAR | ONE_REF | TR_TIME_DOMAIN);

  // Set number of states
  setNumberOfStates(2);
}

void AbmVCann::init() throw(string&)
{
  // create tape
  DenseIntVector var(2,0);
  var[1] = 1;
  DenseIntVector novar;
  DenseDoubleVector nodelay;
  createTape(var, novar, novar, novar, nodelay);
}


void AbmVCann::eval(adoublev& x,
	       adoublev& vp, adoublev& ip)
{
  // x[0]: vin
  // x[1]: vout

  //  Assign voltages directly.
  vp[0] = x[0];
  vp[1] = x[1];
    
  // Use small signal model.
  // Set input current to zero.  
  ip[0] = zero;
      
  // Equation for output current
  if(x[0]>=0)
  		ip[1] = (g*x[0]/pow((1+pow((g/l*x[0]),s)),1/s))/rout;
  else
 		ip[1] = (g*x[0]/pow((1+pow((-g/l*x[0]),s)),1/s))/rout;
}
