// CMOS Inverter
//
//
//                  Vdd 1
//                      o
//                      |
//                      |
//                  |---+
//                  |
//            |----O|
//            |     |
//            |     |---+
//            |         |
// Input 2 o--|         |---o 3 Output
//            |         |
//            |     |---+
//            |     |
//            |-----|
//                  |
//                  |---+
//                      |
//                      |
//                      o
//                  GND 4
//
//
//	  Author:
//		Mazen M Kharbutli
//

#ifndef CmosInv_h
#define CmosInv_h 1

// The CmosInv Class
class CmosInv : public AdolcElement
{
	public:

  CmosInv(const string& iname);

  ~CmosInv() {}

  static const char* getNetlistName()
  {
		return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);

	private:

  // Evaluation function
  virtual void eval(adoublev& x, adoublev& vp, adoublev& ip);

  // Parameter Variables
  double vtn, vtp, un, up, en, ep, tox, ln, wn, lp, wp, td;

  // Model Variables
  double betan, betap;

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif

