// This may look like C code, but it is really -*- C++ -*-
//
// CMOS Inverter with Junction temperature
//
//
//                             Vdd 1
//                                o
//                                 |
//                                 |
//                           |---+
//                           |     
//                  |----O|     
//                  |        |               		  Thermal nodes
//                  |        |---+  
//                  |              |		 	  Junction Temp
// Input 2 o--|              |---o 3 Output		5     
//                  |              |			            o
//                  |       |---+		                       	|
//                  |       |     				|
//                  |-----|       				|
//                          |     				|
//                          |---+				|
//                                |				|	
//                                |				o
//                               o				6
//                          GND 4		     Thermal Ref
//        
//        
//
//	  Authors:
//		 Tony Mulder, Travis Lentz
//

#ifndef CmosInvT_h
#define CmosInvT_h 1

// The CmosInvT Class
class CmosInvT : public AdolcElement
{
public:
  
  CmosInvT(const string& iname);

  ~CmosInvT() {}
  
  static const char* getNetlistName()
  {
     return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);

  virtual void getLocalRefIdx(UnsignedVector& local_ref_vec,
  			      TerminalVector& term_list);



private:

  // Evaluation function
  virtual void eval(adoublev& x, adoublev& vp, adoublev& ip);
  
  // Parameter Variables
  double vtn, vtp, un, up, en, ep, tox, ln, wn, lp, wp, td, ts, tnom, zt;
  double c1, c2, c3, c4, c5, freq, lk;

  // Model Variables
  double betan, betap, power;

  // Element information
  static ItemInfo einfo;
  
  // Number of parameters of this element
  static const unsigned n_par;
  
  bool thermal;       
  
  // Parameter information
  static ParmInfo pinfo[];

};

#endif
