// 
//
//   UCSD HBT model
//                
//
//
//                  0 Collector 
//                  |           
//                  |  
//	           |
//               ||  
//    Base  0----|    
//               |    --------0 Power dissipation
//               ||   --------0 Thermal ground
//                 | 
//                  |
//                  |  
//                  0 Emitter
//
//
//
//	Author: Jian Ding, Sonali Luniya
//		    


#ifndef HBTnpnxT_h
#define HBTnpnxT_h 1

class HBTnpnxT : public AdolcElement2
{
public:

  HBTnpnxT(const string& iname);

  ~HBTnpnxT() {}

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);

  virtual void getLocalRefIdx(UnsignedVector& local_ref_vec,
  			      TerminalVector& term_list);

private:

  // Generic state variable evaluation routine
  virtual void eval1(adoublev& x, adoublev& xt, adoublev& y1, adoublev& z1);
  virtual void eval2(adoublev& dy1, adoublev& z1, adoublev& vp, adoublev& ip);

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter variables
  double type, afn, bf, bfn, bkdn, br, bvc, ccmin, cemin, cjc, cjcx, cje, cjs, cth, cxmin, dtmax,
         eaa, eab, eac, eae, eax, eg, fa, fc, fce, fex, icrit0, ics, ik, ikrk, is, isa, isb, isc,
         iscx, ise, isex, itc, itc2, kfn, mjc, mjcx, mje, mjs, na, nb, nbc, nc, ncs, ncx, ne, nex,
         nf, nr, rbi, rbx, rci, rcx, re, rex, rth, tbcxs, tbexs, tfb, tfc0, tkrk, tnc, tne,
         tnex, tnom, tr, trx, tre, tvjc, tvjci, tvjcx, tvje, tvjs, vaf, var, vjc, vjci, vjcx, vje,
         vjs, vkrk, vtc, xcjc, xrb, xrc, xre, xrex, xrt, xtb, xti, xtikrk, xtitc, xtitc2, xttf, xttkrk, xtvkrk, b, ts;
  double bm1;
  bool kirchhoff;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif
