// Curtice cubic MESFET model (from Microwave Harmonica Manual)
//
//          Drain 2
//                  o
//                  |
//                  |
//              |---+
//              |
// Gate 1 o-----|
//              |
//              |---+
//                  |
//                  |
//                  o
//         Source 3
//
//
//	  Author:
//		  Senthil Velu

#ifndef MesfetCQ_h
#define MesfetCQ_h 1

class MesfetCQ : public AdolcElement
{
	public:

  MesfetCQ(const string& iname);

  ~MesfetCQ() {}

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // Do some local initialization
  virtual void init() throw(string&);

	private:

  virtual void eval(adoublev& x,
	adoublev& vp, adoublev& ip);

  // Some constants
  double k2, k3,M1,M2;
  double delta_T, tn, Vt, k1, k4, k5, k6, Vt0, Beta, Ebarr, EbarrN, Nn;
  double Is, Vbi;
  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter variables
  double a0, a1, a2, a3, beta, vds0, gama, vt0;
  double cgs0, cgd0, is, n, ib0, nr;
  double t, vbi, fcc, vbd, area;
  double tnom, avt0, bvt0, tbet, tm, tme, eg, m, xti, tj;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif

