#include "../../../../network/ElementManager.h"
#include "../../../../analysis/TimeDomainSV.h"
#include "../../../../analysis/FreqDomainSV.h"
#include "../../../../network/AdolcElement.h"
#include "Open.h"

// Static members
const unsigned Open::n_par = 0;

// Element information
ItemInfo Open::einfo =
{
  "open",
  "Open circuit to be used as a voltage meter",
  "Carlos Christoffersen",
  DEFAULT_ADDRESS"category:lumped",
  "2000_07_20"
};

// Parameter information
// ParmInfo Open::pinfo[] = {
//   {"rcomp", "Svtr compensation resistance (Ohms)", DOUBLE, false}
// };

Open::Open(const string& iname) : AdolcElement(&einfo, NULL, n_par, iname)
{

  // Set the number of terminals
  setNumTerms(2);

  // Set flags
  setFlags(NONLINEAR | ONE_REF | TR_TIME_DOMAIN);

  // Set number of states
  setNumberOfStates(1);
  // create tape
  DenseIntVector var(1,0);
  DenseIntVector novar;
  DenseDoubleVector nodelay;
  createTape(var, novar, novar, novar, nodelay);
}

void Open::eval(adoublev& x, adoublev& vp, adoublev& ip)
{
  vp[0] = x[0];
  ip[0] = zero;
}

void Open::svHB(FreqDomainSV* fdsv)
{
  // The voltage is the state variable
  fdsv->getVp(0).resize(fdsv->getX(0).size());
	mtl::copy(fdsv->getX(0), fdsv->getVp(0));
  // The current is always zero
  mtl::set_value(fdsv->getIp(0), zero);
}

void Open::deriv_svHB(FreqDomainSV* fdsv)
{
  mtl::set_value(fdsv->getJacVp(0,0), zero);
  const int& nosamples = fdsv->noSamples();
  for (int i=0; i<nosamples; i++)
    fdsv->getJacVp(0,0)(i,i) = one;
  mtl::set_value(fdsv->getJacIp(0,0), zero);
}

void Open::svTran(TimeDomainSV* tdsv)
{
  tdsv->u(0) = tdsv->getX(0);
  tdsv->i(0) = zero;
}

void Open::deriv_svTran(TimeDomainSV* tdsv)
{
  tdsv->getJu()(0,0) = one;
  tdsv->getJi()(0,0) = zero;
}
