// This may look like C code, but it is really -*- C++ -*-
//
// This is a Voltage controlled current source with delay
//
//                 1                        2
//                  o--------+      +---+--o  ---> Iout
//                                  |    
//           	    +  	            @ - delay  +
//           	  Vin	            |   line   Vout 
//           	    -	           ( )         -
//                                  |    
//           	    o--------+      +---+--o
//                 3                        4
//
// by Frank P. Hart

#ifndef Vccsd_h
#define Vccsd_h 1

class Vccsd : public AdolcElement
{
public:
  
  Vccsd(const string& iname);

  ~Vccsd() {}

  static const char* getNetlistName()
  {
    return einfo.name;
  }

  // This is a multi-reference element.
  virtual void getLocalRefIdx(UnsignedVector& local_ref_vec,
			      TerminalVector& term_list);
  // fill MNAM
//  virtual void fillMNAM(FreqMNAM* mnam);
//  virtual void fillMNAM(TimeMNAM* mnam);
//  virtual void getExtraRC(unsigned& first_eqn, unsigned& n_rows) const ;
//  virtual unsigned getExtraRC(const unsigned& eqn_number, const MNAMType& type) ;
  virtual void eval(adoublev& x, adoublev& vp, adoublev& ip) ;
  virtual void init() throw(string&) ;
private:

  // Element information
  static ItemInfo einfo;

  // Number of parameters of this element
  static const unsigned n_par;

  // Parameter variables
  double n;
  double g;
  double td;

  // Parameter information
  static ParmInfo pinfo[];

};

#endif
