// ----------------------------------------------------------------------
// Main routine for fREEDA
// ----------------------------------------------------------------------

extern "C"
{
  #include <stdlib.h>
  #include <stdio.h>
  #include <time.h>
  #include "./inout/parser.h"
  #include "./inout/report.h"
}

#include <iostream>
#include <cstring>
using namespace std;
#include "./network/CircuitManager.h"
#include "help.h"

void An_DoEndOutput(); // Defined in an_output.c
void anMain(Circuit* cir);  // Defined in anMain.cc

// Global pointer to version string
const char* freeda_version="1.4";

// Global pointer to the main circuit
Circuit* main_cir;

int maimed = 0;

int main(int argc, char **argv)
  {
  processCommandLine(argc, argv);

  // Open input/output files
  Init();

  // Create main circuit
  main_cir = the_CM->getCircuit("Main");

  // Parse the input netlist
  cout << "\n\n   *** Parsing input netlist ...\n\n";
  if (yyparse() || maimed)
    { report(FATAL, "Errors in netlist prevent continuation.\n\n");
    }

  try
    {
    // Print information about netlist if requested. (before expansion)
    dumpOptions();

    // Expand the main circuit (and init elements in expanded subcircuits).
    printf("   *** Expanding subcircuits ...");
    fprintf(output_F, "   *** Expanding subcircuits ...");
    main_cir->expand();
    main_cir->check();
    printf(" done.\n\n");

    // Initialize all circuits
    printf("   *** Initializing and Expanding Elements ...");
    fprintf(output_F, "   *** Initializing and Expanding Elements ...");
    main_cir->init();
    printf(" done.\n\n");

    // Print information about netlist if requested. (output expansion)
    dumpOptions();

    // Check that reference terminals are correct
    printf("   *** Checking reference terminals ...");
    fprintf(output_F, "   *** Checking reference terminals ...");
    main_cir->checkReferences();
    printf(" done.\n\n");
    }
    catch (string& error)
		{
    report(FATAL, error.c_str());
    }

  // Transfer control to the main analysis routine
  anMain(main_cir);

  // Delete the main circuit (the only one after the expansion).
  the_CM->deleteCircuit("Main");

  // Print end time
  {
  time_t curtime;
  char str[90];
  time(&curtime);
  strftime(str, 80, "%c", localtime(&curtime));
  printf("\n**************  fREEDA %s stopping on %s  ***********\n",
		freeda_version, str);
  fprintf(output_F,
	"\n\n**************  fREEDA %s stopping on %s  ***********\n",
	freeda_version, str);
  }

  // Clean all the symbol tables, sweeps and expressions.
  majorCleanUp();
  exit(0);
  }

