// The interface file for Matrix Template Library

#ifndef freeda_mtl_h
#define freeda_mtl_h 1

#include <cstring>
#include <complex>
#include <vector>
#include <cmath>
using namespace std;

#include "mtl/matrix.h"
#include "mtl/mtl.h"
#include "mtl/utils.h"  
#include "mtl/linalg_vec.h"

typedef std::complex<double> double_complex;

// A column-oriented sparse matrix of type double
typedef mtl::matrix< double, 
mtl::rectangle<>, 
mtl::compressed<>,
mtl::column_major>::type DoubleSparseColMatrix; 

// A row-oriented dense matrix of type double
typedef mtl::matrix< double, 
mtl::rectangle<>, 
mtl::dense<>,
mtl::row_major>::type DoubleDenseRowMatrix;

// A column-oriented dense matrix of type double 
typedef mtl::matrix< double, 
mtl::rectangle<>, 
mtl::dense<>,
mtl::column_major>::type DoubleDenseColMatrix; 

// A row-oriented external dense matrix of type double	
typedef mtl::matrix< double, 
mtl::rectangle<>, 
mtl::dense<mtl::external>,
mtl::row_major>::type DoubleDenseExtRowMatrix; 

// A column-oriented external dense matrix of type double
typedef mtl::matrix< double, 
mtl::rectangle<>, 
mtl::dense<mtl::external>,
mtl::column_major>::type DoubleDenseExtColMatrix;

// A column-oriented dense matrix of type int
typedef mtl::matrix< int, 
mtl::rectangle<>, 
mtl::dense<>,
mtl::column_major>::type IntDenseColMatrix; 

// A column-oriented dense matrix of type double_complex
typedef mtl::matrix< double_complex, 
mtl::rectangle<>, 
mtl::dense<>,
mtl::column_major>::type ComplexDenseColMatrix; 	

// A dense vector of type double
typedef mtl::dense1D<double> DenseDoubleVector;

// An external dense vector of type double
typedef mtl::external_vec<double> ExtDoubleVector;

// A dense vector of type int
typedef mtl::dense1D<int> DenseIntVector;

// A dense vector of type double_complex
typedef mtl::dense1D< double_complex > DenseComplexVector;

// An external dense vector of type double_complex
typedef mtl::external_vec< double_complex > ExtComplexVector;

// A sparse vector of type double
typedef mtl::compressed1D<double> CompressedDoubleVector;

// STL vectors
typedef vector<unsigned> UnsignedVector;

// Now declare some handy global constants
extern const double zero;
extern const double one;

extern const double_complex Czero;
extern const double_complex Cone;

extern const double pi;
extern const double twopi;
// Constant to convert degrees to radians (divide for inverse conversion)
extern const double deg2rad;

// Boltzman constant
extern const double kBoltzman;
// Electron charge
extern const double eCharge;
// Permittivity of free space 
extern const double epsilon0;
// Permeability of free space
extern const double mu0;
// Speed of light in free space
extern const double c0;

#endif

